﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Web.Mvc;

using Eco.Handles;
using Eco.Services;
using Eco.Web;


namespace Community.Eco.Web.Mvc
{
    /// <summary>
    /// This class is to provide basic Eco access to those that derive from it.
    /// </summary>
    public class EcoController: System.Web.Mvc.Controller, IDisposable
    {
        // Local variables
        private Type FEcoSpaceType = null;
        private EcoSpace FEcoSpace = null;
        private EcoSpaceStrategyHandler FEcoSpaceProvider = null;


        private void ReturnEcoSpace()
        {
            UpdateDatabase();
            EcoSpaceProvider.ReturnEcoSpace(FEcoSpace);
            FEcoSpace = null;
            FEcoSpaceProvider = null;
        }

        public void UpdateDatabase()
        {
            if (FEcoSpace != null)
            {
                IDirtyListService dls = FEcoSpace.GetEcoService<IDirtyListService>();
                if (dls != null)
                {
                    if (dls.HasDirtyObjects())
                    {
                        IPersistenceService ps = FEcoSpace.GetEcoService<IPersistenceService>();
                        if (ps != null)
                            ps.UpdateDatabaseWithList(dls.AllDirtyObjects());
                    }
                }
            }
        }

        #region IDisposable Implementation
        public new void Dispose()
        {
            Dispose(true);

            base.Dispose();
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                // Can dispose of managed objects
                if (FEcoSpace != null)
                    this.ReturnEcoSpace();
            }
            else
            {
                // Unmanaged only resources can be disposed of now.
            }

            base.Dispose(disposing);
        }
        #endregion

        protected EcoSpaceStrategyHandler EcoSpaceProvider
        {
            get
            {
                if (FEcoSpaceProvider == null)
                {
                    FEcoSpaceProvider = new EcoSpaceStrategyHandler(EcoSpaceStrategyHandler.SessionStateMode.Never, FEcoSpaceType);
                }
                
                return FEcoSpaceProvider;
            }
        }

        public Type EcoSpaceType
        {
            get
            {
                return FEcoSpaceType;
            }

            set
            {

                if (FEcoSpaceType != value)
                    FEcoSpaceType = value;
            }
        }

        public EcoSpace EcoSpace
        {
            get
            {
                if (FEcoSpace == null)
                {
                    if (FEcoSpaceType == null)
                    {
                        throw new ArgumentNullException("EcoSpaceType", "You MUST Set the EcoSpaceType before we can obtain an EcoSpace");
                    }
                    else
                    {
                        FEcoSpace = this.EcoSpaceProvider.GetEcoSpace();
                    }
                }

                return FEcoSpace;
            }
        }
    }
}
