﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Mvc;

using Eco.Services;

using EcoMvcExample.MvcWebsite;
using EcoMvcExample.MvcWebsite.Controllers;
using EcoMvcExample.MvcWebsite.Models;
using EcoMvcExample.MvcWebsite.Tests.Models;
using EcoMvcExample.Model;

using NUnit.Framework;

using Rhino.Mocks;

using System.Security.Principal;


namespace EcoMvcExample.MvcWebsite.Tests.Controllers
{
    /// <summary>
    /// Summary description for HomeControllerTest
    /// </summary>
    [TestFixture]
    public class TasksControllerTest
    {


        [Test]
        public void Index()
        {
            // Setup
            MockRepository mocks = new MockRepository();
            TasksController controller = new TasksController();
            // Here we are going to create some ECO objects that will be used in the Test Memory EcoSpace
            // to be returned on calls to the appropriate controller, which should allow it to do some
            // work, or return exceptions if there are errors.
            controller.EcoSpaceType = typeof(TestMemoryEcoSpace);

            TestObjectsHelper.CreateTestObjects(controller.EcoSpace, true);
            MvcMockHelpers.SetFakeControllerContextWithUser(mocks, controller, "test2");

            mocks.ReplayAll();
            
            // Execute
            ViewResult result = controller.Index() as ViewResult;
            

            // Verify
            IDictionary<string, object> viewData = result.ViewData as IDictionary<string, object>;
            Assert.AreEqual("My Tasks", viewData["Title"]);
            IList<Task> userTasks = result.ViewData.Model as IList<Task>;
            Assert.AreEqual(2, userTasks.Count, "The number of Tasks returned is incorrect!");

            mocks.VerifyAll();
        }

        [Test]
        public void All()
        {
            // Setup
            MockRepository mocks = new MockRepository();
            TasksController controller = new TasksController();
            controller.EcoSpaceType = typeof(TestMemoryEcoSpace);

            TestObjectsHelper.CreateTestObjects(controller.EcoSpace, true);
            MvcMockHelpers.SetFakeControllerContextWithUser(mocks, controller, "test1");

            mocks.ReplayAll();

            // Here we are going to create some ECO objects that will be used in the Test Memory EcoSpace
            // to be returned on calls to the appropriate controller, which should allow it to do some
            // work, or return exceptions if there are errors.

            // Execute
            ViewResult result = controller.All() as ViewResult;

            // Verify
            IDictionary<string, object> viewData = result.ViewData as IDictionary<string, object>;
            Assert.AreEqual("All Tasks", viewData["Title"]);
            IList<Task> userTasks = result.ViewData.Model as IList<Task>;
            Assert.AreEqual(4, userTasks.Count, "The number of Tasks returned is incorrect!");
            
            mocks.VerifyAll();
        }

        [Test]
        public void Create()
        {
            // Setup
            MockRepository mocks = new MockRepository();
            TasksController controller = new TasksController();
            controller.EcoSpaceType = typeof(TestMemoryEcoSpace);

            const string UserThatCanCreateTasks = "test1";
            TestObjectsHelper.CreateTestObjects(controller.EcoSpace, true);
            MvcMockHelpers.SetFakeControllerContextWithUser(mocks, controller, UserThatCanCreateTasks);

            mocks.ReplayAll();

            // Execute
            ViewResult result = controller.Create() as ViewResult;

            // Verify
            IDictionary<string, object> viewData = result.ViewData as IDictionary<string, object>;
            Assert.AreEqual("Create New Task", viewData["Title"]);

            mocks.VerifyAll();
        }

        [Test]
        [ExpectedException()]
        public void CreateInaccessibleForThisUser()
        {
            // Setup
            MockRepository mocks = new MockRepository();
            TasksController controller = new TasksController();
            controller.EcoSpaceType = typeof(TestMemoryEcoSpace);

            const string UserThatCannotCreateTasks = "test2";
            TestObjectsHelper.CreateTestObjects(controller.EcoSpace, true);
            MvcMockHelpers.SetFakeControllerContextWithUser(mocks, controller, UserThatCannotCreateTasks);

            mocks.ReplayAll();

            // Execute
            ViewResult result = controller.Create() as ViewResult;

            // Verify
            IDictionary<string, object> viewData = result.ViewData as IDictionary<string, object>;
            Assert.AreEqual("Create New Task", viewData["Title"]);
        }

        [Test]
        public void Details()
        {
            // Setup
            //TasksController controller = new TasksController();
            //TestMemoryEcoSpace ecoSpace = new TestMemoryEcoSpace();
            //Task newTask = new Task(ecoSpace);
            //ecoSpace.UpdateDatabase();
            const string Username = "test1";
            MockRepository mocks = new MockRepository();
            TasksController controller = new TasksController();
            controller.EcoSpaceType = typeof(TestMemoryEcoSpace);

            TestObjectsHelper.CreateTestObjects(controller.EcoSpace, true);
            MvcMockHelpers.SetFakeControllerContextWithUser(mocks, controller, Username);

            mocks.ReplayAll();

            IList<Task> userTasks = TaskHelper.GetUserTasks(controller.EcoSpace, Username);
            Assert.IsNotNull(userTasks, "userTasks is null!");

            Task task = userTasks[0];

            IExternalIdService eis = controller.EcoSpace.GetEcoService<IExternalIdService>();
            string sID = eis.IdForObject(task.AsIObject());

            // Execute
            ViewResult result = controller.Details(sID) as ViewResult;

            // Verify
            IDictionary<string, object> viewData = result.ViewData as IDictionary<string, object>;
            Assert.AreEqual("View Task", viewData["Title"]);

            Task vdTask = result.ViewData.Model as Task;
            Assert.IsNotNull(vdTask, "ViewData.Model as Task is null!!");
            Assert.AreEqual(task, vdTask);

            mocks.VerifyAll();
        }
    }
}
