﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Eco.ObjectRepresentation;
using Eco.Services;

using EcoMvcExample.Model;


namespace EcoMvcExample.MvcWebsite.Tests.Models
{
    internal static class TestObjectsHelper
    {
        /// <summary>
        /// This method is purely used to create some test objects for evaluating operations
        /// against an EcoSpace.  In this case I have made it a shared/static class, so that
        /// the same code and objects can be reused in many of the tests.
        /// </summary>
        public static void CreateTestObjects(IEcoServiceProvider ecoProvider, bool PersistChanges)
        {
            // Create test task user(s)
            TaskUser tuFirst = new TaskUser(ecoProvider);
            tuFirst.Username = "test1";
            tuFirst.CanCreateTasks = true;

            Task tOne = new Task(ecoProvider);
            tOne.User = tuFirst;
            tOne.Name = "First Test Task";
            tOne.DateEntered = DateTime.Now;
            tOne.Completed = false;

            Task tTwo = new Task(ecoProvider);
            tTwo.User = tuFirst;
            tTwo.Name = "Second Test Task";
            tTwo.DateEntered = DateTime.Now;
            tTwo.Completed = false;


            // Create second test task user
            TaskUser tuSecond = new TaskUser(ecoProvider);
            tuSecond.Username = "test2";
            tuSecond.CanCreateTasks = false;

            Task tThree = new Task(ecoProvider);
            tThree.User = tuSecond;
            tThree.Name = "Third Test Task";
            tThree.DateEntered = DateTime.Now;
            tThree.Completed = false;

            Task tFour = new Task(ecoProvider);
            tFour.User = tuSecond;
            tFour.Name = "Fourth Test Task";
            tFour.DateEntered = DateTime.Now;
            tFour.Completed = true;


            // This allows us to choose to simulate tests done against objects persisted to a data storage
            // or perhaps only against memory loaded objects, to allow specific selection criteria to be
            // tested to suit your needs.  I only include it here as an example of commiting them to persistence.
            if (PersistChanges)
            {
                IDirtyListService dls = ecoProvider.GetEcoService<IDirtyListService>();
                IPersistenceService ps = ecoProvider.GetEcoService<IPersistenceService>();

                if (dls != null)
                {
                    if (dls.HasDirtyObjects())
                    {
                        ps.UpdateDatabaseWithList(dls.AllDirtyObjects());
                    }
                }
            }
        }
    }
}
